@@ -1,7 +1,7 @@
 /* SPDX-License-Identifier: GPL-2.0-or-later */
 
 #include "nvme-print.h"
-#include "util/logging.h"
+#include "logging.h"
 #include "common.h"
 
 static struct print_ops binary_print_ops;
@@ -27,7 +27,7 @@ static void binary_persistent_event_log(void *pevent_log_info,
 }
 
 static void binary_endurance_group_event_agg_log(
-	struct nvme_aggregate_predictable_lat_event *endurance_log,
+	struct nvme_aggregate_endurance_group_event *endurance_log,
 	__u64 log_entries, __u32 size, const char *devname)
 {
 	d_raw((unsigned char *)endurance_log, size);
@@ -428,6 +428,7 @@ static struct print_ops binary_print_ops = {
 	.print_nvme_subsystem_list	= NULL,
 	.topology_ctrl			= NULL,
 	.topology_namespace		= NULL,
+	.topology_multipath		= NULL,
 
 	/* status and error messages */
 	.connect_msg			= NULL,
@@ -435,6 +436,7 @@ static struct print_ops binary_print_ops = {
 	.show_perror			= NULL,
 	.show_status			= NULL,
 	.show_error_status		= NULL,
+	.show_key_value			= NULL,
 };
 
 struct print_ops *nvme_get_binary_print_ops(nvme_print_flags_t flags)
