@@ -3,6 +3,7 @@
 #define NVME_PRINT_H
 
 #include "nvme.h"
+#include "util/table.h"
 #include <inttypes.h>
 
 #include <ccan/list/list.h>
@@ -32,7 +33,7 @@ struct print_ops {
 	void (*directive)(__u8 type, __u8 oper, __u16 spec, __u32 nsid, __u32 result, void *buf, __u32 len);
 	void (*discovery_log)(struct nvmf_discovery_log *log, int numrec);
 	void (*effects_log_list)(struct list_head *list);
-	void (*endurance_group_event_agg_log)(struct nvme_aggregate_predictable_lat_event *endurance_log, __u64 log_entries, __u32 size, const char *devname);
+	void (*endurance_group_event_agg_log)(struct nvme_aggregate_endurance_group_event *endurance_log, __u64 log_entries, __u32 size, const char *devname);
 	void (*endurance_group_list)(struct nvme_id_endurance_group_list *endgrp_list);
 	void (*endurance_log)(struct nvme_endurance_group_log *endurance_group, __u16 group_id, const char *devname);
 	void (*error_log)(struct nvme_error_log_page *err_log, int entries, const char *devname);
@@ -64,6 +65,7 @@ struct print_ops {
 	void (*predictable_latency_event_agg_log)(struct nvme_aggregate_predictable_lat_event *pea_log, __u64 log_entries, __u32 size, const char *devname);
 	void (*predictable_latency_per_nvmset)(struct nvme_nvmset_predictable_lat_log *plpns_log, __u16 nvmset_id, const char *devname);
 	void (*primary_ctrl_cap)(const struct nvme_primary_ctrl_cap *caps);
+	void (*relatives)(nvme_root_t r, const char *name);
 	void (*resv_notification_log)(struct nvme_resv_notification_log *resv, const char *devname);
 	void (*resv_report)(struct nvme_resv_status *status, int bytes, bool eds);
 	void (*sanitize_log_page)(struct nvme_sanitize_log_page *sanitize_log, const char *devname);
@@ -97,13 +99,16 @@ struct print_ops {
 	void (*host_discovery_log)(struct nvme_host_discover_log *log);
 	void (*ave_discovery_log)(struct nvme_ave_discover_log *log);
 	void (*pull_model_ddc_req_log)(struct nvme_pull_model_ddc_req_log *log);
+	void (*log)(const char *devname, struct nvme_get_log_args *args);
 
 	/* libnvme tree print functions */
-	void (*list_item)(nvme_ns_t n);
+	void (*list_item)(nvme_ns_t n, struct table *t);
 	void (*list_items)(nvme_root_t t);
 	void (*print_nvme_subsystem_list)(nvme_root_t r, bool show_ana);
 	void (*topology_ctrl)(nvme_root_t r);
 	void (*topology_namespace)(nvme_root_t r);
+	void (*topology_multipath)(nvme_root_t r);
+	void (*topology_tabular)(nvme_root_t r);
 
 	/* status and error messages */
 	void (*connect_msg)(nvme_ctrl_t c);
@@ -111,6 +116,7 @@ struct print_ops {
 	void (*show_perror)(const char *msg, va_list ap);
 	void (*show_status)(int status);
 	void (*show_error_status)(int status, const char *msg, va_list ap);
+	void (*show_key_value)(const char *key, const char *val, va_list ap);
 
 	nvme_print_flags_t flags;
 };
@@ -152,7 +158,7 @@ struct print_ops *nvme_get_binary_print_ops(nvme_print_flags_t flags);
 
 void nvme_show_status(int status);
 void nvme_show_lba_status_info(__u32 result);
-void nvme_show_relatives(const char *name);
+void nvme_show_relatives(nvme_root_t r, const char *name, nvme_print_flags_t flags);
 
 void nvme_show_id_iocs(struct nvme_id_iocs *iocs, nvme_print_flags_t flags);
 void nvme_show_id_ctrl(struct nvme_id_ctrl *ctrl, nvme_print_flags_t flags,
@@ -198,7 +204,7 @@ void nvme_show_persistent_event_log(void *pevent_log_info,
 	__u8 action, __u32 size, const char *devname,
 	nvme_print_flags_t flags);
 void nvme_show_endurance_group_event_agg_log(
-	struct nvme_aggregate_predictable_lat_event *endurance_log,
+	struct nvme_aggregate_endurance_group_event *endurance_log,
 	__u64 log_entries, __u32 size, const char *devname,
 	nvme_print_flags_t flags);
 void nvme_show_lba_status_log(void *lba_status, __u32 size,
@@ -247,6 +253,7 @@ void nvme_show_list_ns(struct nvme_ns_list *ns_list,
 void nvme_show_topology(nvme_root_t t,
 			enum nvme_cli_topo_ranking ranking,
 			nvme_print_flags_t flags);
+void nvme_show_topology_tabular(nvme_root_t t, nvme_print_flags_t flags);
 
 void nvme_feature_show(enum nvme_features_id fid, int sel, unsigned int result);
 void nvme_feature_show_fields(enum nvme_features_id fid, unsigned int result, unsigned char *buf);
@@ -275,7 +282,7 @@ void nvme_show_zns_report_zones(void *report, __u32 descs,
 				nvme_print_flags_t flags);
 void json_nvme_finish_zone_list(__u64 nr_zones, 
 	struct json_object *zone_list);
-void nvme_show_list_item(nvme_ns_t n);
+void nvme_show_list_item(nvme_ns_t n, struct table *t);
 
 void nvme_show_fdp_configs(struct nvme_fdp_config_log *configs, size_t len,
 		nvme_print_flags_t flags);
@@ -300,6 +307,9 @@ const char *nvme_feature_temp_sel_to_string(__u8 sel);
 const char *nvme_feature_temp_type_to_string(__u8 type);
 const char *nvme_feature_to_string(enum nvme_features_id feature);
 const char *nvme_feature_wl_hints_to_string(__u8 wh);
+const char *nvme_feature_perfc_attri_to_string(__u8 attri);
+const char *nvme_feature_perfc_r4karl_to_string(__u8 r4karl);
+const char *nvme_feature_perfc_attrtyp_to_string(__u8 attrtyp);
 const char *nvme_host_metadata_type_to_string(enum nvme_features_id fid, __u8 type);
 const char *nvme_log_to_string(__u8 lid);
 const char *nvme_nss_hw_error_to_string(__u16 error_code);
@@ -322,6 +332,7 @@ const char *nvme_pel_ehai_pit_to_string(enum nvme_pel_ehai_pit pit);
 const char *nvme_ssi_state_to_string(__u8 state);
 const char *nvme_time_scale_to_string(__u8 ts);
 const char *nvme_pls_mode_to_string(__u8 mode);
+const char *nvme_bpwps_to_string(__u8 bpwps);
 
 void nvme_dev_full_path(nvme_ns_t n, char *path, size_t len);
 void nvme_generic_full_path(nvme_ns_t n, char *path, size_t len);
@@ -330,11 +341,13 @@ void nvme_show_perror(const char *msg, ...);
 void nvme_show_error_status(int status, const char *msg, ...);
 void nvme_show_init(void);
 void nvme_show_finish(void);
+void nvme_show_key_value(const char *key, const char *value, ...);
 bool nvme_is_fabrics_reg(int offset);
 bool nvme_is_fabrics_optional_reg(int offset);
 bool nvme_registers_cmbloc_support(__u32 cmbsz);
 bool nvme_registers_pmrctl_ready(__u32 pmrctl);
 const char *nvme_degrees_string(long t);
+const char *nvme_degrees_fahrenheit_string(long t);
 void print_array(char *name, __u8 *data, int size);
 void json_print(struct json_object *r);
 struct json_object *obj_create_array_obj(struct json_object *o, const char *k);
@@ -352,6 +365,8 @@ void nvme_show_host_discovery_log(struct nvme_host_discover_log *log, nvme_print
 void nvme_show_ave_discovery_log(struct nvme_ave_discover_log *log, nvme_print_flags_t flags);
 void nvme_show_pull_model_ddc_req_log(struct nvme_pull_model_ddc_req_log *log,
 				      nvme_print_flags_t flags);
+void nvme_show_log(const char *devname, struct nvme_get_log_args *args, nvme_print_flags_t flags);
+void nvme_show_pel_header(struct nvme_persistent_event_log *pevent_log_head, int human);
 
 extern char *alloc_error;
 #endif /* NVME_PRINT_H */
