@@ -39,6 +39,7 @@ enum nvme_print_flags {
 	JSON		= 1 << 1,	/* display in json format */
 	VS		= 1 << 2,	/* hex dump vendor specific data areas */
 	BINARY		= 1 << 3,	/* binary dump raw bytes */
+	TABULAR		= 1 << 4,	/* prints aligned columns for easy reading */
 };
 
 typedef uint32_t nvme_print_flags_t;
@@ -46,6 +47,7 @@ typedef uint32_t nvme_print_flags_t;
 enum nvme_cli_topo_ranking {
 	NVME_CLI_TOPO_NAMESPACE,
 	NVME_CLI_TOPO_CTRL,
+	NVME_CLI_TOPO_MULTIPATH,
 };
 
 #define SYS_NVME "/sys/class/nvme"
@@ -78,6 +80,9 @@ struct nvme_config {
 	char *output_format;
 	int verbose;
 	__u32 timeout;
+	bool dry_run;
+	bool no_retries;
+	unsigned int output_format_ver;
 };
 
 /*
@@ -90,6 +95,11 @@ struct nvme_config {
 		OPT_FMT("output-format", 'o', &nvme_cfg.output_format, output_format), \
 		##__VA_ARGS__,                                                         \
 		OPT_UINT("timeout",      't', &nvme_cfg.timeout,       timeout),       \
+		OPT_FLAG("dry-run",        0, &nvme_cfg.dry_run,       dry_run),       \
+		OPT_FLAG("no-retries",     0, &nvme_cfg.no_retries,                    \
+			 "disable retry logic on errors\n"),                           \
+		OPT_UINT("output-format-version", 0, &nvme_cfg.output_format_ver,      \
+			 "output format version: 1|2"),                                \
 		OPT_END()                                                              \
 	}
 
@@ -114,6 +124,18 @@ static inline nvme_mi_ep_t dev_mi_ep(struct nvme_dev *dev)
 	return dev->mi.ep;
 }
 
+static inline bool nvme_is_multipath(nvme_subsystem_t s)
+{
+	nvme_ns_t n;
+	nvme_path_t p;
+
+	nvme_subsystem_for_each_ns(s, n)
+		nvme_namespace_for_each_path(n, p)
+			return true;
+
+	return false;
+}
+
 void register_extension(struct plugin *plugin);
 
 /*
@@ -131,6 +153,7 @@ static inline DEFINE_CLEANUP_FUNC(
 extern const char *output_format;
 extern const char *timeout;
 extern const char *verbose;
+extern const char *dry_run;
 extern struct nvme_config nvme_cfg;
 
 int validate_output_format(const char *format, nvme_print_flags_t *flags);
