@@ -1,11 +1,13 @@
-// SPDX-License-Identifier: GPL-2.0-or-later
-/*
- * Copyright (c) Micron, Inc 2024.
+/* SPDX-License-Identifier: GPL-2.0-or-later */
+/* Copyright (c) Micron, Inc 2024.
  *
  * @file: micron-nvme.h
  * @brief: This module contains all the constructs needed for micron nvme-cli plugin.
- * @authors:Chaithanya Shoba <ashoba@micron.com>,
+ * @authors:Hanumanthu H <hanumanthuh@micron.com>
+ *			Chaithanya Shoba <ashoba@micron.com>
+ *			Sivaprasad Gutha <sivaprasadg@micron.com>
  */
+
 #undef CMD_INC_FILE
 #define CMD_INC_FILE plugins/micron/micron-nvme
 
@@ -35,6 +37,15 @@ PLUGIN(NAME("micron", "Micron vendor specific extensions", NVME_VERSION),
 		ENTRY("vs-smart-add-log", "Retrieve extended SMART data", micron_ocp_smart_health_logs)
 		ENTRY("clear-fw-activate-history", "Clear FW activation history", micron_clr_fw_activation_history)
 		ENTRY("vs-smbus-option", "Enable/Disable SMBUS on the drive", micron_smbus_option)
+		ENTRY("cloud-boot-SSD-version", "Prints HyperScale Boot Version",
+			micron_cloud_boot_SSD_version)
+		ENTRY("vs-device-waf", "Reports SLC and TLC WAF ratio", micron_device_waf)
+		ENTRY("vs-cloud-log",
+			"Retrieve Extended Health Information of Hyperscale NVMe Boot SSD",
+			micron_cloud_log)
+		ENTRY("vs-work-load-log", "Retrieve Workload logs", micron_work_load_log)
+		ENTRY("vs-vendor-telemetry-log",
+			"Retrieve Vendor Telemetry logs", micron_vendor_telemetry_log)
 	)
 );
 
