@@ -22,7 +22,7 @@
 #include "plugin.h"
 #include "linux/types.h"
 #include "util/types.h"
-#include "util/logging.h"
+#include "logging.h"
 #include "nvme-print.h"
 #include "nvme-wrap.h"
 
@@ -189,6 +189,7 @@ struct ieee1667_get_cq_entry {
 
 static const char *sel = "[0-3]: current/default/saved/supported";
 static const char *no_uuid = "Skip UUID index search (UUID index not required for OCP 1.0)";
+static const char *all_ns = "Apply to all namespaces";
 const char *data = "Error injection data structure entries";
 const char *number = "Number of valid error injection data entries";
 static const char *type = "Error injection type";
@@ -423,6 +424,80 @@ int ocp_set_latency_monitor_feature(int argc, char **argv, struct command *cmd,
 	return err;
 }
 
+static int ocp_get_latency_monitor_feature(int argc, char **argv, struct command *cmd,
+					   struct plugin *plugin)
+{
+	const char *desc = "Define Issue Get Feature command (FID: 0xC5) Latency Monitor";
+	const char *sel = "[0-3]: current/default/saved/supported/";
+	const char *nsid = "Byte[04-07]: Namespace Identifier Valid/Invalid/Inactive";
+
+	_cleanup_nvme_dev_ struct nvme_dev *dev = NULL;
+
+	__u32 result;
+	int err;
+	bool uuid;
+	__u8 uuid_index = 0;
+
+	struct config {
+		__u8 sel;
+		__u32 nsid;
+	};
+
+	struct config cfg = {
+		.sel = 0,
+		.nsid = 0,
+	};
+
+	OPT_ARGS(opts) = {
+		OPT_BYTE("sel", 's', &cfg.sel, sel),
+		OPT_UINT("namespace-id", 'n', &cfg.nsid, nsid),
+		OPT_FLAG("no-uuid", 'u', NULL, no_uuid),
+		OPT_END()
+	};
+
+	err = parse_and_open(&dev, argc, argv, desc, opts);
+	if (err)
+		return err;
+
+	uuid = !argconfig_parse_seen(opts, "no-uuid");
+
+	if (uuid) {
+		/* OCP 2.0 requires UUID index support */
+		err = ocp_get_uuid_index(dev, &uuid_index);
+		if (err || !uuid_index) {
+			nvme_show_error("ERROR: No OCP UUID index found");
+			return err;
+		}
+	}
+
+	struct nvme_get_features_args args = {
+		.args_size  = sizeof(args),
+		.fd         = dev_fd(dev),
+		.fid        = OCP_FID_LM,
+		.nsid       = cfg.nsid,
+		.sel        = cfg.sel,
+		.cdw11      = 0,
+		.uuidx      = uuid_index,
+		.data_len   = 0,
+		.data       = NULL,
+		.timeout    = NVME_DEFAULT_IOCTL_TIMEOUT,
+		.result     = &result,
+	};
+
+	err = nvme_get_features(&args);
+	if (!err) {
+		printf("get-feature:0xC5 %s value: %#08x\n",
+		nvme_select_to_string(cfg.sel), result);
+
+		if (cfg.sel == NVME_GET_FEATURES_SEL_SUPPORTED)
+			nvme_show_select_result(0xC5, result);
+	} else {
+		nvme_show_error("Could not get feature: 0xC5");
+	}
+
+	return err;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////////
@@ -1147,6 +1222,12 @@ static int get_telemetry_dump(struct nvme_dev *dev, char *filename, char *sn,
 		size = le16_to_cpu(logheader->DataArea3LastBlock) -
 		       le16_to_cpu(logheader->DataArea2LastBlock);
 		break;
+	case 4:
+		offset = TELEMETRY_HEADER_SIZE +
+			 (le16_to_cpu(logheader->DataArea3LastBlock) * TELEMETRY_BYTE_PER_BLOCK);
+		size = le16_to_cpu(logheader->DataArea4LastBlock) -
+			   le16_to_cpu(logheader->DataArea3LastBlock);
+		break;
 	default:
 		break;
 	}
@@ -1163,13 +1244,15 @@ static int get_telemetry_dump(struct nvme_dev *dev, char *filename, char *sn,
 	return err;
 }
 
-static int get_telemetry_log_page_data(struct nvme_dev *dev, int tele_type)
+static int get_telemetry_log_page_data(struct nvme_dev *dev,
+		int tele_type,
+		int tele_area,
+		const char *output_file)
 {
-	char file_path[PATH_MAX];
 	void *telemetry_log;
 	const size_t bs = 512;
 	struct nvme_telemetry_log *hdr;
-	size_t full_size, offset = bs;
+	size_t full_size = 0, offset = bs;
 	int err, fd;
 
 	if ((tele_type == TELEMETRY_TYPE_HOST_0) || (tele_type == TELEMETRY_TYPE_HOST_1))
@@ -1188,11 +1271,10 @@ static int get_telemetry_log_page_data(struct nvme_dev *dev, int tele_type)
 	}
 	memset(hdr, 0, bs);
 
-	sprintf(file_path, DEFAULT_TELEMETRY_BIN);
-	fd = open(file_path, O_WRONLY | O_CREAT | O_TRUNC, 0666);
+	fd = open(output_file, O_WRONLY | O_CREAT | O_TRUNC, 0666);
 	if (fd < 0) {
 		fprintf(stderr, "Failed to open output file %s: %s!\n",
-			file_path, strerror(errno));
+				output_file, strerror(errno));
 		err = fd;
 		goto exit_status;
 	}
@@ -1230,9 +1312,25 @@ static int get_telemetry_log_page_data(struct nvme_dev *dev, int tele_type)
 		goto close_fd;
 	}
 
-	full_size = (le16_to_cpu(hdr->dalb3) * bs) + offset;
+	switch (tele_area) {
+	case 1:
+		full_size = (le16_to_cpu(hdr->dalb1) * bs) + offset;
+		break;
+	case 2:
+		full_size = (le16_to_cpu(hdr->dalb2) * bs) + offset;
+		break;
+	case 3:
+		full_size = (le16_to_cpu(hdr->dalb3) * bs) + offset;
+		break;
+	case 4:
+		full_size = (le32_to_cpu(hdr->dalb4) * bs) + offset;
+		break;
+	default:
+		full_size = offset;
+		break;
+	}
 
-	while (offset != full_size) {
+	while (offset < full_size) {
 		args.log = telemetry_log;
 		args.lpo = offset;
 		args.lsp = NVME_LOG_LSP_NONE;
@@ -1264,14 +1362,16 @@ exit_status:
 	return err;
 }
 
-static int get_c9_log_page_data(struct nvme_dev *dev, int print_data, int save_bin)
+static int get_c9_log_page_data(struct nvme_dev *dev,
+		int print_data,
+		int save_bin,
+		const char *output_file)
 {
 	int ret = 0;
 	__le64 stat_id_str_table_ofst = 0;
 	__le64 event_str_table_ofst = 0;
 	__le64 vu_event_str_table_ofst = 0;
 	__le64 ascii_table_ofst = 0;
-	char file_path[PATH_MAX];
 
 	_cleanup_fd_ int fd = STDIN_FILENO;
 
@@ -1330,10 +1430,9 @@ static int get_c9_log_page_data(struct nvme_dev *dev, int print_data, int save_b
 	}
 
 	if (save_bin) {
-		sprintf(file_path, DEFAULT_STRING_BIN);
-		fd = open(file_path, O_WRONLY | O_CREAT | O_TRUNC, 0666);
+		fd = open(output_file, O_WRONLY | O_CREAT | O_TRUNC, 0666);
 		if (fd < 0) {
-			fprintf(stderr, "Failed to open output file %s: %s!\n", file_path,
+			fprintf(stderr, "Failed to open output file %s: %s!\n", output_file,
 				strerror(errno));
 			return fd;
 		}
@@ -1356,7 +1455,7 @@ int parse_ocp_telemetry_log(struct ocp_telemetry_parse_options *options)
 
 	if (options->telemetry_log) {
 		if (strstr((const char *)options->telemetry_log, "bin")) {
-			// Read the data from the telemetry binary file
+			/* Read the data from the telemetry binary file */
 			ptelemetry_buffer =
 				read_binary_file(NULL, (const char *)options->telemetry_log,
 						 &telemetry_buffer_size, 1);
@@ -1377,7 +1476,7 @@ int parse_ocp_telemetry_log(struct ocp_telemetry_parse_options *options)
 	}
 
 	if (options->string_log) {
-		// Read the data from the string binary file
+		/* Read the data from the string binary file */
 		if (strstr((const char *)options->string_log, "bin")) {
 			pstring_buffer = read_binary_file(NULL, (const char *)options->string_log,
 							  &string_buffer_size, 1);
@@ -1408,11 +1507,15 @@ static int ocp_telemetry_log(int argc, char **argv, struct command *cmd, struct
 	const char *telemetry_log = "Telemetry log binary;\n 'host.bin' or 'controller.bin'";
 	const char *string_log = "String log binary; 'C9.bin'";
 	const char *output_file = "Output file name with path;\n"
-			"e.g. '-o ./path/name'\n'-o ./path1/path2/';\n"
+			"e.g. '-f ./path/name'\n'-f ./path1/path2/';\n"
 			"If requested path does not exist, the directory will be newly created.";
 	const char *output_format = "output format normal|json";
-	const char *data_area = "Telemetry Data Area; 1 or 2;\n"
-			"e.g. '-a 1 for Data Area 1.'\n'-a 2 for Data Areas 1 and 2.';\n";
+	const char *data_area = "Telemetry Data Area; 1, 2, 3, or 4;\n"
+			"e.g. '-a 1 for Data Area 1.'\n"
+			"e.g. '-a 2 for Data Areas 1 and 2.'\n"
+			"e.g. '-a 3 for Data Areas 1, 2, and 3.'\n"
+			"e.g. '-a 4 for Data Areas 1, 2, 3, and 4.';\n";
+
 	const char *telemetry_type = "Telemetry Type; 'host', 'host0', 'host1' or 'controller'";
 
 	struct nvme_dev *dev;
@@ -1425,12 +1528,16 @@ static int ocp_telemetry_log(int argc, char **argv, struct command *cmd, struct
 	struct ocp_telemetry_parse_options opt;
 	int tele_type = 0;
 	int tele_area = 0;
+	char file_path_telemetry[PATH_MAX], file_path_string[PATH_MAX];
+	const char *string_suffix = "string.bin";
+	const char *tele_log_suffix = "telemetry.bin";
+	bool host_behavior_changed = false;
 
 	OPT_ARGS(opts) = {
 		OPT_STR("telemetry-log", 'l', &opt.telemetry_log, telemetry_log),
 		OPT_STR("string-log", 's', &opt.string_log, string_log),
-		OPT_FILE("output-file", 'o', &opt.output_file, output_file),
-		OPT_FMT("output-format", 'f', &opt.output_format, output_format),
+		OPT_FILE("output-file", 'f', &opt.output_file, output_file),
+		OPT_FMT("output-format", 'o', &opt.output_format, output_format),
 		OPT_INT("data-area", 'a', &opt.data_area, data_area),
 		OPT_STR("telemetry-type", 't', &opt.telemetry_type, telemetry_type),
 		OPT_END()
@@ -1461,11 +1568,15 @@ static int ocp_telemetry_log(int argc, char **argv, struct command *cmd, struct
 
 	is_support_telemetry_controller = ((ctrl.lpa & 0x8) >> 3);
 
+	if (opt.output_file == NULL)
+		opt.output_file = DEFAULT_TELEMETRY_LOG;
+
 	if (!opt.data_area) {
 		nvme_show_result("Missing data-area. Using default data area 1.\n");
 		opt.data_area = DATA_AREA_1;//Default data area 1
-	} else if (opt.data_area != 1 && opt.data_area != 2) {
-		nvme_show_result("Invalid data-area specified. Please specify 1 or 2.\n");
+	} else if (opt.data_area != 1 && opt.data_area != 2 &&
+			   opt.data_area != 3 && opt.data_area != 4) {
+		nvme_show_result("Invalid data-area specified. Please specify 1, 2, 3, or 4.\n");
 		goto out;
 	}
 
@@ -1493,24 +1604,55 @@ static int ocp_telemetry_log(int argc, char **argv, struct command *cmd, struct
 
 	if (!opt.telemetry_log) {
 		nvme_show_result("\nMissing telemetry-log. Fetching from drive...\n");
-		err = get_telemetry_log_page_data(dev, tele_type);//Pull Telemetry log
+
+		if (tele_area == 4) {
+			if (!(ctrl.lpa & 0x40)) {
+				nvme_show_error("Telemetry data area 4 not supported by device.\n");
+				goto out;
+			}
+
+			err = nvme_set_etdas(dev_fd(dev), &host_behavior_changed);
+			if (err) {
+				fprintf(stderr, "%s: Failed to set ETDAS bit\n", __func__);
+				return err;
+			}
+		}
+
+		/* Pull the Telemetry log */
+		sprintf(file_path_telemetry, "%s-%s", opt.output_file, tele_log_suffix);
+		err = get_telemetry_log_page_data(dev,
+				tele_type,
+				tele_area,
+				(const char *)file_path_telemetry);
 		if (err) {
 			nvme_show_error("Failed to fetch telemetry-log from the drive.\n");
 			goto out;
 		}
 		nvme_show_result("telemetry.bin generated. Proceeding with next steps.\n");
-		opt.telemetry_log = DEFAULT_TELEMETRY_BIN;
+		opt.telemetry_log = file_path_telemetry;
+
+		if (host_behavior_changed) {
+			host_behavior_changed = false;
+			err = nvme_clear_etdas(dev_fd(dev), &host_behavior_changed);
+			if (err) {
+				/* Continue on if this fails, it's not a fatal condition */
+				nvme_show_error("Failed to clear ETDAS bit.\n");
+			}
+		}
 	}
 
 	if (!opt.string_log) {
 		nvme_show_result("Missing string-log. Fetching from drive...\n");
-		err = get_c9_log_page_data(dev, 0, 1); //Pull String log
+
+		/* Pull String log  */
+		sprintf(file_path_string, "%s-%s", opt.output_file, string_suffix);
+		err = get_c9_log_page_data(dev, 0, 1, (const char *)file_path_string);
 		if (err) {
 			nvme_show_error("Failed to fetch string-log from the drive.\n");
 			goto out;
 		}
 		nvme_show_result("string.bin generated. Proceeding with next steps.\n");
-		opt.string_log = DEFAULT_STRING_BIN;
+		opt.string_log = file_path_string;
 	}
 
 	if (!opt.output_format) {
@@ -1521,6 +1663,7 @@ static int ocp_telemetry_log(int argc, char **argv, struct command *cmd, struct
 	switch (tele_type) {
 	case TELEMETRY_TYPE_HOST:
 		printf("Extracting Telemetry Host Dump (Data Area %d)...\n", tele_area);
+
 		err = parse_ocp_telemetry_log(&opt);
 		if (err)
 			nvme_show_result("Status:(%x)\n", err);
@@ -1948,6 +2091,85 @@ static int ocp_set_telemetry_profile_feature(int argc, char **argv, struct comma
 	return err;
 }
 
+///////////////////////////////////////////////////////////////////////////////
+///////////////////////////////////////////////////////////////////////////////
+///////////////////////////////////////////////////////////////////////////////
+///////////////////////////////////////////////////////////////////////////////
+/// DSSD Power State (Feature Identifier C8h) Get Feature
+static int ocp_get_telemetry_profile_feature(int argc, char **argv, struct command *cmd,
+					      struct plugin *plugin)
+{
+	const char *desc = "Define Issue Get Feature command (FID: 0xC8) Latency Monitor";
+	const char *sel = "[0-3]: current/default/saved/supported/";
+	const char *nsid = "Byte[04-07]: Namespace Identifier Valid/Invalid/Inactive";
+
+	_cleanup_nvme_dev_ struct nvme_dev *dev = NULL;
+
+	__u32 result;
+	int err;
+	bool uuid;
+	__u8 uuid_index = 0;
+
+	struct config {
+		__u8 sel;
+		__u32 nsid;
+	};
+
+	struct config cfg = {
+		.sel = 0,
+		.nsid = 0,
+	};
+
+	OPT_ARGS(opts) = {
+		OPT_BYTE("sel", 's', &cfg.sel, sel),
+		OPT_UINT("namespace-id", 'n', &cfg.nsid, nsid),
+		OPT_FLAG("no-uuid", 'u', NULL, no_uuid),
+		OPT_END()
+	};
+
+	err = parse_and_open(&dev, argc, argv, desc, opts);
+	if (err)
+		return err;
+
+	uuid = !argconfig_parse_seen(opts, "no-uuid");
+
+	if (uuid) {
+		/* OCP 2.0 requires UUID index support */
+		err = ocp_get_uuid_index(dev, &uuid_index);
+		if (err || !uuid_index) {
+			nvme_show_error("ERROR: No OCP UUID index found");
+			return err;
+		}
+	}
+
+	struct nvme_get_features_args args = {
+		.args_size  = sizeof(args),
+		.fd         = dev_fd(dev),
+		.fid        = OCP_FID_TEL_CFG,
+		.nsid       = cfg.nsid,
+		.sel        = cfg.sel,
+		.cdw11      = 0,
+		.uuidx      = uuid_index,
+		.data_len   = 0,
+		.data       = NULL,
+		.timeout    = NVME_DEFAULT_IOCTL_TIMEOUT,
+		.result     = &result,
+	};
+
+	err = nvme_get_features(&args);
+	if (!err) {
+		printf("get-feature:0xC8 %s value: %#08x\n",
+		nvme_select_to_string(cfg.sel), result);
+
+		if (cfg.sel == NVME_GET_FEATURES_SEL_SUPPORTED)
+			nvme_show_select_result(0xC8, result);
+	} else {
+		nvme_show_error("Could not get feature: 0xC8");
+	}
+
+	return err;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////////
 ///////////////////////////////////////////////////////////////////////////////
@@ -2421,7 +2643,9 @@ static int get_dssd_async_event_config(int argc, char **argv, struct command *cm
 static int ocp_telemetry_str_log_format(int argc, char **argv, struct command *cmd,
 					struct plugin *plugin);
 
-static int get_c9_log_page(struct nvme_dev *dev, char *format)
+static int get_c9_log_page(struct nvme_dev *dev,
+		char *format,
+		const char *output_file)
 {
 	int ret = 0;
 	nvme_print_flags_t fmt;
@@ -2432,16 +2656,12 @@ static int get_c9_log_page(struct nvme_dev *dev, char *format)
 		return ret;
 	}
 
-	if (fmt == BINARY)
-		ret = get_c9_log_page_data(dev, 0, 1);
-	else
-		ret = get_c9_log_page_data(dev, 0, 0);
+	ret = get_c9_log_page_data(dev, 0, 1, output_file);
 
-	if (!ret) {
+	if ((!ret) && (fmt != BINARY))
 		ocp_c9_log(log_data, pC9_string_buffer, total_log_page_sz, fmt);
-	} else {
+	else if (ret)
 		fprintf(stderr, "ERROR : OCP : Unable to read C9 data from buffer\n");
-	}
 
 	free(header_data);
 	return ret;
@@ -2452,19 +2672,27 @@ static int ocp_telemetry_str_log_format(int argc, char **argv, struct command *c
 {
 	struct nvme_dev *dev;
 	int ret = 0;
+	char file_path[PATH_MAX];
+	const char *string_suffix = ".bin";
 	const char *desc = "Retrieve telemetry string log format";
+	const char *output_file = "Output file name with path;\n"
+			"e.g. '-f ./path/name'\n'-f ./path1/path2/';\n"
+			"If requested path does not exist, the directory will be newly created.";
 
 	struct config {
 		char *output_format;
+		char *output_file;
 	};
 
 	struct config cfg = {
 		.output_format = "normal",
+		.output_file = NULL,
 	};
 
 	OPT_ARGS(opts) = {
 		OPT_FMT("output-format", 'o', &cfg.output_format,
 				"output Format:normal|json|binary"),
+		OPT_FILE("output-file", 'f', &cfg.output_file, output_file),
 		OPT_END()
 	};
 
@@ -2472,7 +2700,12 @@ static int ocp_telemetry_str_log_format(int argc, char **argv, struct command *c
 	if (ret)
 		return ret;
 
-	ret = get_c9_log_page(dev, cfg.output_format);
+	if (cfg.output_file != NULL)
+		sprintf(file_path, "%s-%s", cfg.output_file, string_suffix);
+	else
+		sprintf(file_path, "%s", DEFAULT_STRING_BIN);
+
+	ret = get_c9_log_page(dev, cfg.output_format, file_path);
 	if (ret)
 		fprintf(stderr, "ERROR : OCP : Failure reading the C9 Log Page, ret = %d\n", ret);
 
@@ -2612,13 +2845,19 @@ static int clear_pcie_correctable_error_counters(int argc, char **argv, struct c
 	return ocp_clear_pcie_correctable_errors(argc, argv, cmd, plugin);
 }
 
+static int get_clear_pcie_correctable_error_counters(int argc, char **argv, struct command *cmd,
+						      struct plugin *plugin)
+{
+	return get_ocp_error_counters(argc, argv, cmd, plugin);
+}
+
 static int fw_activation_history_log(int argc, char **argv, struct command *cmd,
 				     struct plugin *plugin)
 {
 	return ocp_fw_activation_history_log(argc, argv, cmd, plugin);
 }
 
-static int error_injection_get(struct nvme_dev *dev, const __u8 sel, bool uuid)
+static int error_injection_get(struct nvme_dev *dev, const __u8 sel, bool uuid, __u32 nsid)
 {
 	struct erri_get_cq_entry cq_entry;
 	int err;
@@ -2629,7 +2868,7 @@ static int error_injection_get(struct nvme_dev *dev, const __u8 sel, bool uuid)
 
 	struct nvme_get_features_args args = {
 		.result = (__u32 *)&cq_entry,
-		.data = entry,
+		.nsid = nsid,
 		.args_size = sizeof(args),
 		.fd = dev_fd(dev),
 		.timeout = NVME_DEFAULT_IOCTL_TIMEOUT,
@@ -2652,22 +2891,23 @@ static int error_injection_get(struct nvme_dev *dev, const __u8 sel, bool uuid)
 		nvme_show_error("malloc: %s", strerror(errno));
 		return -errno;
 	}
+	args.data = entry;
 
 	err = nvme_cli_get_features(dev, &args);
 	if (!err) {
-		nvme_show_result("Number of Error Injecttions (feature: %#0*x): %#0*x (%s: %d)",
+		nvme_show_result("Number of Error Injections (feature: %#0*x): %#0*x (%s: %d)",
 				 fid ? 4 : 2, fid, cq_entry.nume ? 10 : 8, cq_entry.nume,
 				 nvme_select_to_string(sel), cq_entry.nume);
 		if (sel == NVME_GET_FEATURES_SEL_SUPPORTED)
 			nvme_show_select_result(fid, *args.result);
 		for (i = 0; i < cq_entry.nume; i++) {
 			printf("Entry: %d, Flags: %x (%s%s), Type: %x (%s), NRTDP: %d\n", i,
-			       entry->flags, entry->enable ? "Enabled" : "Disabled",
-			       entry->single ? ", Single instance" : "", entry->type,
-			       erri_type_to_string(entry->type), entry->nrtdp);
+			       entry[i].flags, entry[i].enable ? "Enabled" : "Disabled",
+			       entry[i].single ? ", Single instance" : "", entry[i].type,
+			       erri_type_to_string(entry[i].type), entry[i].nrtdp);
 		}
 	} else {
-		nvme_show_error("Could not get feature: %#0*x.", fid ? 4 : 2, fid);
+		nvme_show_error("Could not get feature: %#0*x. %d", fid ? 4 : 2, fid);
 	}
 
 	return err;
@@ -2680,6 +2920,7 @@ static int get_error_injection(int argc, char **argv, struct command *cmd, struc
 	struct config {
 		__u8 sel;
 	};
+	__u32 nsid;
 	struct config cfg = { 0 };
 
 	_cleanup_nvme_dev_ struct nvme_dev *dev = NULL;
@@ -2687,6 +2928,7 @@ static int get_error_injection(int argc, char **argv, struct command *cmd, struc
 	OPT_ARGS(opts) = {
 		OPT_BYTE("sel", 's', &cfg.sel, sel),
 		OPT_FLAG("no-uuid", 'n', NULL, no_uuid),
+		OPT_FLAG("all-ns", 'a', NULL, all_ns),
 		OPT_END()
 	};
 
@@ -2694,15 +2936,24 @@ static int get_error_injection(int argc, char **argv, struct command *cmd, struc
 	if (err)
 		return err;
 
-	return error_injection_get(dev, cfg.sel, !argconfig_parse_seen(opts, "no-uuid"));
+	/*
+	 * Different spec versions ask for different nsid values
+	 * OCP v1.0 - NSID: Shall be set to zero
+	 * OCP v2.0r21 - NSID: Shall be set to FFFFFFFFh.
+	 * OCP v2.5 - NSID: The host should either clear this to zero or set this to FFFFFFFFh
+	 */
+	nsid = argconfig_parse_seen(opts, "all-ns") ? NVME_NSID_ALL : 0;
+
+	return error_injection_get(dev, cfg.sel, !argconfig_parse_seen(opts, "no-uuid"), nsid);
 }
 
-static int error_injection_set(struct nvme_dev *dev, struct erri_config *cfg, bool uuid)
+static int error_injection_set(struct nvme_dev *dev, struct erri_config *cfg, bool uuid, __u32 nsid)
 {
 	int err;
 	__u32 result;
 	struct nvme_set_features_args args = {
 		.args_size = sizeof(args),
+		.nsid = nsid,
 		.fd = dev_fd(dev),
 		.fid = OCP_FID_ERRI,
 		.cdw11 = cfg->number,
@@ -2772,6 +3023,7 @@ static int set_error_injection(int argc, char **argv, struct command *cmd, struc
 {
 	const char *desc = "Inject error conditions";
 	int err;
+	__u32 nsid;
 	struct erri_config cfg = {
 		.number = 1,
 	};
@@ -2782,6 +3034,7 @@ static int set_error_injection(int argc, char **argv, struct command *cmd, struc
 		  OPT_FILE("data", 'd', &cfg.file, data),
 		  OPT_BYTE("number", 'n', &cfg.number, number),
 		  OPT_FLAG("no-uuid", 'N', NULL, no_uuid),
+		  OPT_FLAG("all-ns", 'a', NULL, all_ns),
 		  OPT_SHRT("type", 't', &cfg.type, type),
 		  OPT_SHRT("nrtdp", 'r', &cfg.nrtdp, nrtdp));
 
@@ -2789,7 +3042,14 @@ static int set_error_injection(int argc, char **argv, struct command *cmd, struc
 	if (err)
 		return err;
 
-	return error_injection_set(dev, &cfg, !argconfig_parse_seen(opts, "no-uuid"));
+	/*
+	 * Different spec versions ask for different nsid values
+	 * OCP v1.0 - NSID: Shall be set to zero
+	 * OCP v2.0r21 - NSID: Shall be set to FFFFFFFFh.
+	 * OCP v2.5 - NSID: The host should either clear this to zero or set this to FFFFFFFFh
+	 */
+	nsid = argconfig_parse_seen(opts, "all-ns") ? NVME_NSID_ALL : 0;
+	return error_injection_set(dev, &cfg, !argconfig_parse_seen(opts, "no-uuid"), nsid);
 }
 
 static int enable_ieee1667_silo_get(struct nvme_dev *dev, const __u8 sel, bool uuid)
