@@ -73,9 +73,8 @@ typedef struct _UtilsTimeInfo
 int wdc_UtilsSnprintf(char *buffer, unsigned int sizeOfBuffer, const char *format, ...);
 void wdc_UtilsDeleteCharFromString(char* buffer, int buffSize, char charToRemove);
 int wdc_UtilsGetTime(PUtilsTimeInfo timeInfo);
-int wdc_UtilsStrCompare(char *pcSrc, char *pcDst);
-int wdc_UtilsCreateDir(char *path);
-int wdc_WriteToFile(char *fileName, char *buffer, unsigned int bufferLen);
+int wdc_UtilsStrCompare(const char *pcSrc, const char *pcDst);
+int wdc_UtilsCreateDir(const char *path);
+int wdc_WriteToFile(const char *fileName, const char *buffer, unsigned int bufferLen);
 void wdc_StrFormat(char *formatter, size_t fmt_sz, char *tofmt, size_t tofmtsz);
 bool wdc_CheckUuidListSupport(struct nvme_dev *dev, struct nvme_id_uuid_list *uuid_list);
-bool wdc_UuidEqual(struct nvme_id_uuid_list_entry *entry1, struct nvme_id_uuid_list_entry *entry2);
