@@ -29,8 +29,6 @@ NVMe Namespace Management Testcase:-
     5. Delete Namespace.
 """
 
-import time
-
 from nvme_test import TestNVMe
 
 
@@ -59,7 +57,6 @@ class TestNVMeAttachDetachNSCmd(TestNVMe):
         self.setup_log_dir(self.__class__.__name__)
         self.ctrl_id = self.get_ctrl_id()
         self.delete_all_ns()
-        time.sleep(1)
 
     def tearDown(self):
         """
