@@ -37,9 +37,9 @@ Namespace Format testcase :-
            - Delete Namespace.
 """
 
+import json
 import math
 import subprocess
-import time
 
 from nvme_test import TestNVMe
 
@@ -55,9 +55,7 @@ class TestNVMeFormatCmd(TestNVMe):
               - nsze : namespace size.
               - ncap : namespace capacity.
               - ctrl_id : controller id.
-              - lba_format_list : lis of supported format.
-              - ms_list : list of metadat size per format.
-              - lbads_list : list of LBA data size per format.
+              - lba_format_list : json list of supported format.
               - test_log_dir : directory for logs, temp files.
     """
 
@@ -74,12 +72,9 @@ class TestNVMeFormatCmd(TestNVMe):
         self.nsze = ncap
         self.ctrl_id = self.get_ctrl_id()
         self.lba_format_list = []
-        self.ms_list = []
-        self.lbads_list = []
         self.test_log_dir = self.log_dir + "/" + self.__class__.__name__
         self.setup_log_dir(self.__class__.__name__)
         self.delete_all_ns()
-        time.sleep(1)
 
     def tearDown(self):
         """
@@ -106,50 +101,44 @@ class TestNVMeFormatCmd(TestNVMe):
                                                      self.dps), 0)
         self.assertEqual(self.attach_ns(self.ctrl_id, self.default_nsid), 0)
         # read lbaf information
-        id_ns = "nvme id-ns " + self.ctrl + \
-                " -n1 | grep ^lbaf | awk '{print $2}' | tr -s \"\\n\" \" \""
-        proc = subprocess.Popen(id_ns, shell=True, stdout=subprocess.PIPE,
+        id_ns_cmd = f"{self.nvme_bin} id-ns {self.ctrl} " + \
+            f"--namespace-id={self.default_nsid} --output-format=json"
+        proc = subprocess.Popen(id_ns_cmd,
+                                shell=True,
+                                stdout=subprocess.PIPE,
                                 encoding='utf-8')
-        self.lba_format_list = proc.stdout.read().strip().split(" ")
-        if proc.wait() == 0:
-            # read lbads information
-            id_ns = "nvme id-ns " + self.ctrl + \
-                    " -n1 | grep ^lbaf | awk '{print $5}'" + \
-                    " | cut -f 2 -d ':' | tr -s \"\\n\" \" \""
-            proc = subprocess.Popen(id_ns, shell=True, stdout=subprocess.PIPE,
-                                    encoding='utf-8')
-            self.lbads_list = proc.stdout.read().strip().split(" ")
-            # read metadata information
-            id_ns = "nvme id-ns " + self.ctrl + \
-                    " -n1 | grep ^lbaf | awk '{print $4}'" + \
-                    " | cut -f 2 -d ':' | tr -s \"\\n\" \" \""
-            proc = subprocess.Popen(id_ns, shell=True, stdout=subprocess.PIPE,
-                                    encoding='utf-8')
-            self.ms_list = proc.stdout.read().strip().split(" ")
-            self.assertEqual(self.detach_ns(self.ctrl_id, self.default_nsid), 0)
-            self.assertEqual(self.delete_and_validate_ns(self.default_nsid), 0)
-            self.nvme_reset_ctrl()
+        err = proc.wait()
+        self.assertEqual(err, 0, "ERROR : nvme id-ns failed")
+        json_output = json.loads(proc.stdout.read())
+        self.lba_format_list = json_output['lbafs']
+        self.assertTrue(len(self.lba_format_list) > 0,
+                        "ERROR : nvme id-ns could not find any lba formats")
+        self.assertEqual(self.detach_ns(self.ctrl_id, self.default_nsid), 0)
+        self.assertEqual(self.delete_and_validate_ns(self.default_nsid), 0)
+        self.nvme_reset_ctrl()
 
     def test_format_ns(self):
         """ Testcase main """
         # extract the supported format information.
         self.attach_detach_primary_ns()
 
+        print("##### Testing lba formats:")
         # iterate through all supported format
-        for i in range(0, len(self.lba_format_list)):
-            print("\nlba format " + str(self.lba_format_list[i]) +
-                  " lbad       " + str(self.lbads_list[i]) +
-                  " ms         " + str(self.ms_list[i]))
-            metadata_size = 1 if self.ms_list[i] == '8' else 0
+        for flbas, lba_format in enumerate(self.lba_format_list):
+            ds = lba_format['ds']
+            ms = lba_format['ms']
+            print(f"\nlba format {str(flbas)}"
+                  f"\nds         {str(ds)}"
+                  f"\nms         {str(ms)}")
+            dps = 1 if str(ms) == '8' else 0
             err = self.create_and_validate_ns(self.default_nsid,
                                               self.nsze,
                                               self.ncap,
-                                              self.lba_format_list[i],
-                                              metadata_size)
+                                              flbas,
+                                              dps)
             self.assertEqual(err, 0)
             self.assertEqual(self.attach_ns(self.ctrl_id, self.default_nsid), 0)
-            self.run_ns_io(self.default_nsid, self.lbads_list[i])
-            time.sleep(5)
+            self.run_ns_io(self.default_nsid, int(ds))
             self.assertEqual(self.detach_ns(self.ctrl_id, self.default_nsid), 0)
             self.assertEqual(self.delete_and_validate_ns(self.default_nsid), 0)
             self.nvme_reset_ctrl()
